@@ -5,8 +5,10 @@ Feature selection of structured (tabular) features
 """
 
 import dask.dataframe as dd
+import mlflow
 import os
 import pandas as pd
+import warnings
 
 from .feature_tournament import FeatureTournament
 from .sampler import MLSampler
@@ -16,6 +18,11 @@ from easyexplore.data_visualizer import DataVisualizer
 from sklearn.feature_selection import SelectFromModel
 from typing import Dict, List, Union
 
+warnings.filterwarnings('ignore', category=DeprecationWarning)
+warnings.filterwarnings('ignore', category=FutureWarning)
+warnings.filterwarnings('ignore', category=UserWarning)
+warnings.filterwarnings('ignore', category=RuntimeWarning)
+
 
 class FeatureSelectorException(Exception):
     """
@@ -37,6 +44,7 @@ class FeatureSelector:
                  visualize_all_scores: bool = True,
                  visualize_variant_scores: bool = True,
                  visualize_core_feature_scores: bool = True,
+                 mlflow_log: bool = True,
                  path: str = None,
                  **kwargs
                  ):
@@ -64,6 +72,9 @@ class FeatureSelector:
         :param visualize_core_feature_scores: bool
             Whether to visualize summarized core feature importance scores or not
 
+        :param mlflow_log: bool
+            Track experiment results using mlflow
+
         :param path: str
             Path or directory to export visualization to
 
@@ -100,6 +111,13 @@ class FeatureSelector:
             else:
                 if self.path.split('/')[-1] != '':
                     self.path = '{}/'.format(self.path)
+        self.mlflow_log: bool = mlflow_log
+        if self.mlflow_log:
+            self.mlflow_client: mlflow.tracking.MlflowClient = mlflow.tracking.MlflowClient(tracking_uri=kwargs.get('tracking_uri'),
+                                                                                            registry_uri=kwargs.get('registry_uri')
+                                                                                            )
+        else:
+            self.mlflow_client = None
         self.kwargs: dict = kwargs
         self.init_pairs: int = self.kwargs.get('init_pairs')
         self.init_games: int = self.kwargs.get('init_games')
@@ -108,17 +126,39 @@ class FeatureSelector:
         self.penalty_factor: float = self.kwargs.get('penalty_factor')
         self.evolutionary_algorithm: str = self.kwargs.get('evolutionary_algorithm')
         self.max_iter = self.kwargs.get('max_iter')
+        self.max_players: int = self.kwargs.get('max_players')
         self.kwargs.pop('init_pairs', None)
         self.kwargs.pop('init_games', None)
         self.kwargs.pop('increasing_pair_size_factor', None)
         self.kwargs.pop('games', None)
         self.kwargs.pop('evolutionary_algorithm', None)
         self.kwargs.pop('max_iter', None)
+        self.kwargs.pop('max_players', None)
+
+    @staticmethod
+    def _mlflow_tracking(stats: Dict[str, pd.DataFrame], file_paths: List[str]):
+        """
+        Track model performance using mlflow
+        """
+        for i, stat in enumerate(stats.keys()):
+            _df: pd.DataFrame = stats.get(stat)
+            mlflow.set_experiment(experiment_name=stat, experiment_id=None)
+            for case in range(0, _df.shape[0], 1):
+                with mlflow.start_run():
+                    for feature in range(0, _df.shape[1], 1):
+                        mlflow.set_tag(key=_df.columns.tolist()[feature], value=_df.iloc[case, feature])
+            with mlflow.start_run():
+                try:
+                    _file_name: str = file_paths[i].split('/')[-1].replace('.html', '')
+                    mlflow.log_artifact(local_path=file_paths[i], artifact_path=_file_name)
+                except (FileNotFoundError, IndexError):
+                    pass
 
     def get_imp_features(self,
                          meth: str = 'shapley',
                          model: str = 'cat',
                          imp_threshold: float = 0.01,
+                         visualize_game_stats: bool = True,
                          plot_type: str = 'bar'
                          ) -> dict:
         """
@@ -129,7 +169,7 @@ class FeatureSelector:
                 -> rf: Random Forest for multi-scaled features
                 -> xgb: Extreme Gradient Boosting Decision Tree for multi-scaled features
                 -> lasso: Lasso Regression for continuous features
-                -> pca: Principle Component Analysis for continuous features
+                -> pca: Principal Component Analysis for continuous features
                 -> shapley: Shapley Value approximation using feature tournament framework
 
         :param model: str
@@ -138,6 +178,9 @@ class FeatureSelector:
         :param imp_threshold: float
             Threshold of importance score
 
+        :param visualize_game_stats: bool
+            Whether to visualize game statistics or not
+
         :param plot_type: str
             Name of the plot type
                 -> pie: Pie Chart
@@ -164,6 +207,8 @@ class FeatureSelector:
                                                   penalty_factor=0.1 if self.penalty_factor is None else self.penalty_factor,
                                                   evolutionary_algorithm='si' if self.evolutionary_algorithm is None else self.evolutionary_algorithm,
                                                   max_iter=50 if self.max_iter is None else self.max_iter,
+                                                  max_players=-1 if self.max_players is None else self.max_players,
+                                                  mlflow_log=False if self.kwargs.get('mlflow_log_shapley') is None else self.kwargs.get('mlflow_log_shapley'),
                                                   **self.kwargs
                                                   ).play()
             _df: pd.DataFrame = pd.DataFrame(data=_imp_scores.get('total'), index=['score']).transpose()
@@ -187,25 +232,34 @@ class FeatureSelector:
             #_game_df['game'] = _game_df.index.values
             _tournament_df: pd.DataFrame = pd.DataFrame(data=_imp_scores.get('tournament'))
             #_tournament_df['game'] = _tournament_df.index.values
+            _file_paths: List[str] = []
             if self.visualize_all_scores:
-                _imp_plot: dict = {'Feature Tournament Game Stats (Shapley Scores)': dict(data=_game_df,
-                                                                                          features=list(_game_df.columns),
-                                                                                          plot_type='violin',
-                                                                                          melt=True,
-                                                                                          render=True,
-                                                                                          file_path='{}feature_tournament_game_stats.html'.format(self.path) if self.path is not None else None
-                                                                                          ),
-                                   'Feature Tournament Stats (Game Size)': dict(data=_tournament_df,
-                                                                                features=list(_tournament_df.columns),
-                                                                                #color_feature='game',
-                                                                                plot_type='heat',
-                                                                                render=True,
-                                                                                file_path='{}feature_tournament_game_size.html'.format(self.path) if self.path is not None else None
-                                                                                ),
-                                   'Feature Importance (Shapley Scores)': dict(df=_df,
+                if visualize_game_stats:
+                    _file_paths.append(os.path.join(str(self.path), 'feature_tournament_game_stats.html'))
+                    _file_paths.append(os.path.join(str(self.path), 'feature_tournament_game_size.html'))
+                    _game_plot: dict = {'Feature Tournament Game Stats (Shapley Scores)': dict(data=_game_df,
+                                                                                               features=list(_game_df.columns),
+                                                                                               plot_type='violin',
+                                                                                               melt=True,
+                                                                                               render=True,
+                                                                                               file_path=_file_paths[0] if self.path is not None else None
+                                                                                               ),
+                                        'Feature Tournament Stats (Game Size)': dict(data=_tournament_df,
+                                                                                     features=list(_tournament_df.columns),
+                                                                                     plot_type='heat',
+                                                                                     render=True,
+                                                                                     file_path=_file_paths[1] if self.path is not None else None
+                                                                                     )
+                                        }
+                    DataVisualizer(subplots=_game_plot,
+                                   height=500,
+                                   width=500
+                                   ).run()
+                _file_paths.append(os.path.join(str(self.path), 'feature_importance_shapley.html'))
+                _imp_plot: dict = {'Feature Importance (Shapley Scores)': dict(df=_df,
                                                                                plot_type=plot_type,
                                                                                render=True if self.path is None else False,
-                                                                               file_path='{}feature_importance_shapley.html'.format(self.path) if self.path is not None else None,
+                                                                               file_path=_file_paths[-1] if self.path is not None else None,
                                                                                kwargs=dict(layout={},
                                                                                            y=_df['score'].values,
                                                                                            x=_df.index.values.tolist(),
@@ -216,12 +270,25 @@ class FeatureSelector:
                                                                                            )
                                                                                )
                                    }
+            if self.mlflow_log:
+                self._mlflow_tracking(stats={'Feature Score (Feature Tournament)': _df,
+                                             'Game Size (Feature Tournament)': _tournament_df,
+                                             'Game Score (Feature Tournament)': _game_df
+                                             },
+                                      file_paths=_file_paths)
         elif meth == 'pca':
-            raise NotImplementedError('Feature selection using Principle Component Analysis (PCA) not implemented')
+            raise NotImplementedError('Feature selection using Principal Component Analysis (PCA) not implemented')
         elif meth in ['lasso', 'cat', 'rf', 'gbo', 'xgb']:
+            if self.mlflow_log:
+                _meth: str = 'Lasso Regression' if meth == 'lasso' else 'Decision Tree'
+                mlflow.set_experiment(experiment_name=f'Feature Selector {_meth}',
+                                      experiment_id=None
+                                      )
             if self.ml_type == 'reg':
                 _ml = ModelGeneratorReg(model_name=meth).generate_model()
             else:
+                if meth == 'lasso':
+                    raise FeatureSelectorException('Cannot perform classification using Lasso Regression model')
                 _ml = ModelGeneratorClf(model_name=meth).generate_model()
             _train_test_data: dict = MLSampler(df=self.df, target=self.target, features=self.features, stratification=False).train_test_sampling()
             _ml.train(x=_train_test_data.get('x_train').values, y=_train_test_data.get('y_train').values)
